@@ -13,7 +13,6 @@
 static jammo_theme* current_theme; //One theme is handled at a time.
 
 static ClutterActor* view_to_go_when_end; //After start this points themeselection or cupboard
-static ClutterActor* container_for_save_menu = NULL; //This contains static elements from json, and dynamic contet from theme
 static gboolean only_playing_mode=FALSE; //When editing this is FALSE. When coming from cupboard, only listen is allowed
 
 static gboolean already_quitted = FALSE; //Pressing cross when quitting has already started.
@@ -28,6 +27,15 @@ void composition_game_close(){
 
 	only_playing_mode=FALSE; //for next round
 
+	//Hide the saving menu
+	clutter_actor_hide(jammo_get_actor_by_id("saving-dialog-view"));
+	//When user selects icon for saved game, cancel is hided. Make it visible again
+	// Because container is hidden, it will not pop on the screen now (but for next round)
+	// If user close game with empty track, there are no actor "saving_dialog_cancel".
+	ClutterActor* saving_dialog_cancel = jammo_get_actor_by_id("saving_dialog_cancel");
+	if (saving_dialog_cancel)
+		clutter_actor_show(saving_dialog_cancel);
+
 	ClutterActor* view = jammo_get_actor_by_id("compositiongame-view");
 	tangle_actor_show(TANGLE_ACTOR(view_to_go_when_end));
 	tangle_actor_hide_animated(TANGLE_ACTOR(view));
@@ -38,7 +46,7 @@ void composition_game_close(){
 
 
 static void hide_save_menu(){
-	clutter_actor_hide(container_for_save_menu);
+	clutter_actor_hide(jammo_get_actor_by_id("saving-dialog-view"));
 	theme_play_all_icon_animations(current_theme);
 }
 
@@ -50,6 +58,8 @@ void composition_game_quit_without_saving_clicked (TangleButton* tanglebutton, g
 void composition_game_back_to_game_clicked (TangleButton* tanglebutton, gpointer none){
 	printf("CANCEL\n");
 	hide_save_menu();
+	//technically we haven't changed the view, so mentor_idle_speech is not changed automatically
+	tangle_action_execute(TANGLE_ACTION(jammo_get_actor_by_id("compostiongame-mentor-action")));
 }
 
 
@@ -87,11 +97,7 @@ void composition_game_cross_clicked (TangleButton* tanglebutton, gpointer none){
 
 	if (pop_menu){
 		theme_stop_all_icon_animations(current_theme);
-		//printf("show current_theme->save_menu_container\n");
-		clutter_actor_raise_top(container_for_save_menu);
-		clutter_actor_show(container_for_save_menu);
-		jammo_mentor_speak(jammo_mentor_get_default(),"composing_select_icon.ogg");
-		jammo_mentor_set_idle_speech(jammo_mentor_get_default(),"composing_select_icon.ogg");
+		clutter_actor_show(jammo_get_actor_by_id("saving-dialog-view"));
 	}
 	else
 		composition_game_close();
@@ -103,7 +109,7 @@ void composition_game_play_clicked (TangleButton* tanglebutton, gpointer none){
 	//If user press play-button when first sample is playing (triggering mentor to give instructions for dragging)
 	//Mentor jumps and talks sametime than track.
 	//This fixes it:
-	jammo_mentor_speak_once(jammo_mentor_get_default(), "composing_first_click.ogg");
+	jammo_mentor_speak_once(jammo_mentor_get_default(), "composing_first_click.spx");
 	jammo_mentor_shut_up(jammo_mentor_get_default());
 
 	//tanglebutton is toggle-button.
@@ -207,12 +213,8 @@ void composition_game_start_theme(const gchar* name, int variation, ClutterActor
 	//Add samplebuttons and track_views
 	clutter_container_add(CLUTTER_CONTAINER(view), current_theme->container, NULL);
 
-	//Add save_menu icons and hide save-menu
-	container_for_save_menu =  jammo_get_actor_by_id("composition_game-container_for_save_icon_menu");
-	clutter_actor_hide(container_for_save_menu);
-	clutter_actor_raise_top(container_for_save_menu);
-	clutter_container_add(CLUTTER_CONTAINER(container_for_save_menu), current_theme->save_menu_container, NULL);
-
-	//When user has used save-menu, this cancel button is hidden. This will not show it because container is still hidden.
-	clutter_actor_show(jammo_get_actor_by_id("composition_game_cancel_saving"));
+	//Add save_menu icons to and hide save-menu
+	ClutterActor* saving_dialog = jammo_get_actor_by_id("saving-dialog-view");
+	clutter_actor_raise_top(saving_dialog);
+	clutter_container_add(CLUTTER_CONTAINER(saving_dialog), current_theme->save_menu_container, NULL); //TODO: tangle_widget
 }
