@@ -172,7 +172,7 @@ static void say_now_you_hear_own_singing(JammoMentor* mentor, const gchar*speech
 		return;
 	}
 
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "singinggame_listen.ogg", start_sequencer, sequencer);
+	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "singinggame_listen.spx", start_sequencer, sequencer);
 }
 
 
@@ -247,7 +247,7 @@ void on_sequencer_stopped_recording(JammoSequencer* sequencer_old, gpointer user
 
 	chum_set_state_of_chum(SCREEN_PLAY_RECORDED);
 
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "welldone.ogg", say_now_you_hear_own_singing, sequencer);
+	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "welldone.spx", say_now_you_hear_own_singing, sequencer);
 }
 
 
@@ -266,7 +266,7 @@ static void say_this_too(JammoMentor* mentor, const gchar*speech, gboolean inter
 }
 
 
-//static
+
 void singinggame_start_songselection(){
 	printf("Singinggame_start_songselection\n");
 	
@@ -284,6 +284,29 @@ void singinggame_start_songselection(){
 	gfloat height_of_container=0;
 	const char* selected_language = chum_get_selected_language();
 
+	//////////////
+	//Change flag to corresponding language
+	ClutterActor* button;
+	ClutterActor* texture;
+	gchar *new_name = g_strdup_printf("%s/language_flag%s.png", DATA_DIR, selected_language); //name contains prefix _
+
+	printf("looking for '%s'\n",new_name);
+	button = jammo_get_actor_by_id("songselection-language");
+	//Remove old texture
+	tangle_button_set_normal_background_actor(TANGLE_BUTTON(button), NULL);
+
+	//And put new
+	texture =  clutter_texture_new_from_file(new_name, NULL);
+	if (texture==NULL) {
+		g_free(new_name);
+		gchar *new_name = g_strdup_printf("%s/language_icon.png", DATA_DIR); //Default icon
+		texture =  clutter_texture_new_from_file(new_name, NULL);
+	}
+	tangle_button_set_normal_background_actor(TANGLE_BUTTON(button), texture);
+
+	g_free(new_name);
+	////////////
+
 	int column_counter=0;  //placement on x-direction
 	int row_counter=0;      //placement on y-direction
 
@@ -408,17 +431,17 @@ void singinggame_start_songselection(){
 	clutter_actor_set_size(container, 672.0, height_of_container);
 
 	//Say this regardless of skill level
-	jammo_mentor_speak_once(jammo_mentor_get_default(),"songselection_welcome.ogg");
+	jammo_mentor_speak_once(jammo_mentor_get_default(),"songselection_welcome.spx");
 
 	if (height_of_container>480) //tell more instructions ( about scrolling )
 		{ 
-			jammo_mentor_speak_once(jammo_mentor_get_default(),"songselection_welcome_advanced.ogg");
+			jammo_mentor_speak_once(jammo_mentor_get_default(),"songselection_welcome_advanced.spx");
 		}
 
 	if (chum_get_duetto())
-		jammo_mentor_set_idle_speech_with_callback(jammo_mentor_get_default(),"songselection_idle.ogg",say_this_too,"songselection_idle_vocal.ogg");
+		jammo_mentor_set_idle_speech_with_callback(jammo_mentor_get_default(),"songselection_idle.spx",say_this_too,"songselection_idle_vocal.spx");
 	else
-		jammo_mentor_set_idle_speech_with_callback(jammo_mentor_get_default(),"songselection_idle.ogg",say_this_too,"songselection_idle_instrument.ogg");
+		jammo_mentor_set_idle_speech_with_callback(jammo_mentor_get_default(),"songselection_idle.spx",say_this_too,"songselection_idle_instrument.spx");
 
 }
 
@@ -427,7 +450,7 @@ static gulong handler_for_mentor_say_selected;
 static void mentor_say_what_is_selected(TangleActor *actor, ClutterActorBox *arg1, gpointer user_data){
 	//printf("got '%s'\n",(gchar*)user_data);
 	gchar *filepath1;
-	filepath1 =  g_strdup_printf("%s_selected.ogg", (gchar*)user_data);
+	filepath1 =  g_strdup_printf("%s_selected.spx", (gchar*)user_data);
 
 	jammo_mentor_speak(jammo_mentor_get_default(), filepath1);
 	g_free(filepath1);
@@ -473,15 +496,6 @@ void singinggame_language_clicked(TangleAction *action, GObject *source, const g
 	chum_set_selected_language(name);
 
 	ClutterActor* view = jammo_get_actor_by_id("songselection-view");
-
-	ClutterActor* button;
-	ClutterActor* texture;
-	gchar *new_name = g_strdup_printf("songselection-language-%s-texture", name);
- 
-	button = jammo_get_actor_by_id("songselection-language");
-	texture = jammo_get_actor_by_id(new_name);
-	tangle_button_set_normal_background_actor(TANGLE_BUTTON(button), texture);
-
 	/*
 	There aren't substring(begin,end) in g_
 	e.g.:
@@ -498,8 +512,6 @@ void singinggame_language_clicked(TangleAction *action, GObject *source, const g
 	name_for_mentor = g_strreverse (g_strndup (g_strreverse(name_for_mentor1),2));
 	g_free(name_for_mentor1);
 	handler_for_mentor_say_selected = g_signal_connect(view, "show-completed", G_CALLBACK(mentor_say_what_is_selected), (gpointer)name_for_mentor);
-
-	g_free(new_name);
 }
 
 
