@@ -98,7 +98,7 @@ static void say_start_after_prelude(JammoMentor* mentor, const gchar*speech, gbo
 		return;
 	}
 
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "songstartC.ogg", start_sequencer, sequencer);
+	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "songstartC.spx", start_sequencer, sequencer);
 }
 
 
@@ -110,7 +110,7 @@ static void say_name_of_song(JammoMentor* mentor, const gchar*speech, gboolean i
 
 	jammo_song* song = chum_get_current_song();
 	gchar *filepath1;
-	filepath1 =  g_strdup_printf("%s/name%s.ogg", song->fullpath, chum_get_selected_language());
+	filepath1 =  g_strdup_printf("%s/name%s.spx", song->fullpath, chum_get_selected_language());
 	//printf("pass to mentor: '%s' \n",filepath1);
 
 	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), filepath1, say_start_after_prelude, sequencer);
@@ -119,9 +119,30 @@ static void say_name_of_song(JammoMentor* mentor, const gchar*speech, gboolean i
 
 static void song_view_showed(ClutterTimeline* t, gpointer sequencer){
 	printf("song_view_showed\n");
-
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "songstartA.ogg", say_name_of_song, sequencer);
 	jammo_mentor_set_idle_speech(jammo_mentor_get_default(), "");
+
+	//If song doesn't have "name_XX.spx" look for "intro_XX.spx"
+	//And then it replaces all other mentor's speaches on this step.
+	jammo_song* song = chum_get_current_song();
+	gchar *filepath1;
+	filepath1 =  g_strdup_printf("%s/name%s.spx", song->fullpath, chum_get_selected_language());
+	int fd;
+	fd = open(filepath1,O_RDONLY);
+	if (fd == -1) //File not found
+		{
+		close(fd);
+		g_free(filepath1);
+		filepath1 =  g_strdup_printf("%s/intro%s.spx", song->fullpath, chum_get_selected_language());
+		fd = open(filepath1,O_RDONLY);
+		if (fd == -1) //File not found
+			{ //No name_XX nor intro_XX, just start sequencer.
+			start_sequencer(NULL,NULL,FALSE,sequencer);
+			}
+		else  //say intro_XX and after that start sequencer
+			jammo_mentor_speak_with_callback(jammo_mentor_get_default(), filepath1, start_sequencer, sequencer);
+		}
+	else //say "you have selected" + name_XX + "start after prelude" and then start sequencer
+		jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "songstartA.spx", say_name_of_song, sequencer);
 }
 
 static void start_sequencer_after_animation(ClutterTimeline* t, gpointer sequencer){
@@ -405,12 +426,12 @@ static gboolean song_icon_clicked(TangleButton* tanglebutton, gpointer data){
 
 		gchar *filename;
 		int fd;
-		filename = g_strdup_printf("%s/comping%s.ogg", song->fullpath,song->language);
+		filename = g_strdup_printf("%s/comping%s.spx", song->fullpath,song->language);
 		fd = open(filename,O_RDONLY);
 		if (fd == -1) //File not found
 			{  //no track for this language,. use  common comping. THIS IS ERROR case. malfuncted savefile!
 			g_free(filename);
-			filename = g_strdup_printf("%s/comping.ogg", song->fullpath); //try without language postfix
+			filename = g_strdup_printf("%s/comping.spx", song->fullpath); //try without language postfix
 			}
 		printf("looking for file '%s' for comping_track\n",filename);
 
