@@ -70,12 +70,12 @@ static gboolean save_image_pressed(TangleButton *tanglebutton, gpointer data){
 	jammo_theme* theme = (jammo_theme*)data;
 	const gchar* filename = clutter_actor_get_name(CLUTTER_ACTOR(tanglebutton));
 
-	clutter_actor_hide(jammo_get_actor_by_id("composition_game_cancel_saving"));
+	clutter_actor_hide(jammo_get_actor_by_id("saving_dialog_cancel")); //No canceling after image is selected.
 
 
 	printf("Selected '%s'\n",filename);
 	composition_game_file_helper_save_composition((gchar*)filename, theme->bottom_track_view, theme->upper_track_view, theme->prefix_for_saving);
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(),"save_confirmed.ogg",animate_icon_to_cupboard,tanglebutton);
+	jammo_mentor_speak_with_callback(jammo_mentor_get_default(),"save_confirmed.spx",animate_icon_to_cupboard,tanglebutton);
 	return FALSE;
 }
 
@@ -164,7 +164,7 @@ static ClutterActor* soundfamily[4]; //This can be GList, but now we know that w
 /*
 actor is button, not interested.
 event is not interested.
-user_data is container/warehouse.
+user_data is container
 */
 static void menu_clicked (TangleButton *tanglebutton, gpointer user_data){
 	//check if this menu is clicked already (is it visible)
