@@ -2,20 +2,25 @@
  * This file is part of JamMo.
  *
  * Instrument_gui is realtime 'jamming' UI.
+ * This is used with midi-instrument and slider.
+ *
+ *
+ * Midi and slider are not symmetric.
+ * Midi-track comes inside of miditrack-view and it should handle visualized notes
+ * Slider-track comes inside of TangeWidget and doesn't need any visualization (at current)
+ *
+ *Three cases:
+ * A) not yet recording, just testing jamming
+ *    there are ad-hoc sequencer containing ad-hoc-instrument-track
+ *B) recording.
+ *    this plays real-sequencer + real-instrument-track on recording mode
+ * c) playback.
+ *    this plays real_sequencer
+ *
+ *
+ * When playbacking static_sequencer, keyboard and slider is not working
+ * i.e. no visual feedback, no audible feedback, no storing events.
  *
- * This is used with midi-instrument and slider. TODO:slider
-
-
-
-Three cases:
-A) not yet recording, just testing jamming
-   there are ad-hoc sequencer containing ad-hoc-instrument-track
-B) recording.
-   this plays real-sequencer + real-instrument-track on recording mode
-c) playback.
-   this plays real_sequencer
-
-
  * (c) 2010 University of Oulu
  *
  * Authors: Aapo Rantalainen
@@ -26,6 +31,7 @@ c) playback.
 #include <ctype.h>
 #include <string.h>
 #include <clutter/clutter.h>
+#include <math.h>
 
 #include "../../meam/jammo-meam.h"
 #include "../../meam/jammo-instrument-track.h"
@@ -37,18 +43,83 @@ c) playback.
 
 #include "midi_editor.h"
 
+
+//For keyboard
 static int UPPER_OCTAVE=3;
+// TRANSPOSE is used to transpose keyboard to a different key
+// e.g. value 2 transposes keyboard from c to d
+// meaning that keys produce sounds d,d#,e... although the keyboard layout is normal c,c#,d...
+static int TRANSPOSE=0;
+void instrument_gui_set_transpose(char key) {
+	switch (key) {
+		case 'c':
+			TRANSPOSE=0;
+			break;
+		case 'a':
+			TRANSPOSE=+9;
+			break;
+		case 'd':
+			TRANSPOSE=+2;
+			break;
+		case 'g':
+			TRANSPOSE=+7;
+			break;
+		default:
+			// discard
+			break;
+	}
+}
+char instrument_gui_get_transpose() {
+	switch (TRANSPOSE) {
+		case 0:
+			return 'c';
+			break;
+		case 2:
+			return 'd';
+			break;
+		case 9:
+			return 'a';
+			break;
+		case 7:
+			return 'g';
+			break;
+		default:
+			// should never come here
+			break;
+	}
+	return 'c';
+}
 
+//For slider
+#define RANGE 2.0
+// X coordinates of playing area
+#define RIGHT_X 599.0
+#define LEFT_X 0.0
+#define OCTAVE_ON_SCREEN ((RIGHT_X - LEFT_X)/RANGE)
+#define BASE_FREQUENCY 130.81
+
+//These we get as parameter and will return after use
 static JammoSequencer* static_sequencer;
+static JammoMiditrackView* static_jammo_miditrack_view;
+static JammoSliderTrack* static_slider_track;
 
+
+//These are made only for this session and should be free'ed
 static JammoInstrumentTrack *ad_hoc_instrument_track;
+static JammoSliderTrack* ad_hoc_slider_track;
 static JammoSequencer* ad_hoc_sequencer;
 
 
+//State of recording
 static gboolean recording;
 
-static JammoMiditrackView* static_jammo_miditrack_view;
-//static JammoSliderTrack *slider_track; TODO.
+//This is pointer to marker of slider (we need to change which slider-track is moving it)
+static ClutterActor* slider_marker;
+
+static void end_jamming();
+
+//Keyboard specific
+//-----------------
 
 /*
 We got notevalue 0...96
@@ -72,11 +143,13 @@ static void visualize_playback(JammoInstrumentTrack* track, gint note, gpointer
 		note *=-1;
 		}
 
+	note -= TRANSPOSE;
+
 	const char* notename = jammomidi_note_to_char(note);
 	if (notename[1]=='#')
 		sharp=TRUE;
 
-	if (note>=12*UPPER_OCTAVE)
+	if (note<=12*UPPER_OCTAVE)
 		bottom=TRUE;
 
 	const char* id  = g_strdup_printf("instrument_key_%c%s%s",notename[0],sharp?"s":"",bottom?"2":"");
@@ -142,7 +215,7 @@ static char notename_to_numericvalue(const gchar* name) {
 	note[1] = '\0';
 
 	if (name[2]!='2') //bottom row
-		octave--;
+		octave++;
 
   //this returns notevalue 0...96
 	char playable_note = jammomidi_char_to_note(note,octave); 
@@ -154,11 +227,10 @@ static char notename_to_numericvalue(const gchar* name) {
 }
 
 /*
-Actually this gets all notifys, but we are using only 'interaction'
+This is JSON-callback for 'notify::interaction'
 i.e pressing and releasing
 */
 void instrument_key_interaction_notify (GObject *object, GParamSpec *pspec, gpointer user_data)  {
-	if (!strcmp(g_param_spec_get_name(pspec), "interacting")){
 		const gchar* name = clutter_actor_get_name(CLUTTER_ACTOR(object));
 		printf("Keyboard '%s' pressed\n",name);
 		TangleButton* button;
@@ -172,15 +244,72 @@ void instrument_key_interaction_notify (GObject *object, GParamSpec *pspec, gpoi
 		}
 
 		button = TANGLE_BUTTON(object);
+		char notevalue = notename_to_numericvalue(name);
+		notevalue += TRANSPOSE;
 		if (tangle_actor_get_interacting(TANGLE_ACTOR(button))) {
-			jammo_instrument_track_note_on_realtime (instrument_track, notename_to_numericvalue(name));
+			jammo_instrument_track_note_on_realtime (instrument_track, notevalue);
 		} else {
-			jammo_instrument_track_note_off_realtime (instrument_track, notename_to_numericvalue(name));
+			jammo_instrument_track_note_off_realtime (instrument_track, notevalue);
 		}
-	}
 
 }
 
+void instrument_goeditor_clicked (TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
+	printf("on_go_editor_button_clicked\n");
+	end_jamming();
+
+	//start editor
+	midi_editor_start_with_miditrack_view(static_jammo_miditrack_view, static_sequencer);
+}
+
+//////////////////////////
+
+
+//slider-specific
+//---------------
+static void update_marker(JammoSliderTrack* track, gfloat frequency, gpointer none) {
+		printf("update_marker: Frequency %f\n",frequency);
+		gfloat x = OCTAVE_ON_SCREEN * log2(frequency/BASE_FREQUENCY);
+		if (frequency==0) //-> x=-inf
+			x=0.0;
+		printf("X: %f\n",x);
+		clutter_actor_set_x(CLUTTER_ACTOR(slider_marker),x);
+}
+
+gboolean instrument_gui_slider_handler (ClutterActor *actor, ClutterEvent *event, gpointer user_data) {
+	gfloat x_release, y_release;
+	gfloat freq;
+	clutter_event_get_coords (event, &x_release, &y_release);
+	//printf("slider_handler: X=%d\n",(gint)x_release);
+
+	freq = BASE_FREQUENCY*pow(2.0, x_release/OCTAVE_ON_SCREEN);
+	//printf("Freq: %f\n",freq);
+
+	JammoSliderTrack* slider_track;
+	if (recording){
+		slider_track = static_slider_track;
+	}
+	else {
+		slider_track = ad_hoc_slider_track;
+	}
+
+
+	if (event->type == CLUTTER_BUTTON_PRESS) {
+		//printf("pressed\n");
+		jammo_slider_track_set_on_realtime(slider_track, freq);
+	} else if (event->type == CLUTTER_BUTTON_RELEASE) {
+		//printf("released\n");
+		jammo_slider_track_set_off_realtime(slider_track, freq);
+	}	else if (event->type == CLUTTER_MOTION) {
+		//printf("motion\n");
+		jammo_slider_track_set_frequency_realtime(slider_track, freq);
+	}
+	return FALSE;
+}
+/////////////////////////////////
+
+
+
 
 
 
@@ -199,13 +328,6 @@ static void end_jamming() {
 	clutter_container_foreach(CLUTTER_CONTAINER(mainview), CLUTTER_CALLBACK(tangle_actor_hide_animated), NULL);
 }
 
-void instrument_goeditor_clicked (TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
-	printf("on_go_editor_button_clicked\n");
-	end_jamming();
-
-	//start editor
-	midi_editor_start_with_miditrack_view(static_jammo_miditrack_view, static_sequencer);
-}
 
 
 void instrument_gosequencer_clicked (TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
@@ -224,6 +346,11 @@ void instrument_play_clicked (TangleAction *action, GObject *source, const gchar
 	//stop ad_hoc.
 	jammo_sequencer_stop (JAMMO_SEQUENCER (ad_hoc_sequencer));
 
+	if (static_slider_track) {
+		//Change this slider_track to move slider_marker.
+		g_signal_connect (static_slider_track, "report-current-frequency", G_CALLBACK (update_marker),NULL);
+	}
+
 	//Start real-sequencer
 	jammo_sequencer_play(JAMMO_SEQUENCER(static_sequencer));
 }
@@ -246,14 +373,30 @@ void instrument_record_clicked (TangleButton *tanglebutton, gpointer user_data)
 		//stop ad_hoc.
 		jammo_sequencer_stop (JAMMO_SEQUENCER (ad_hoc_sequencer));
 
+		//Slider:
+		if (static_jammo_miditrack_view==NULL) {
+			//clean original
+			jammo_slider_track_set_event_list(JAMMO_SLIDER_TRACK(static_slider_track),NULL);
 
-		//clean original
-		JammoTrack* track;
-		g_object_get(static_jammo_miditrack_view,"track",&track,NULL);
-		jammo_instrument_track_set_event_list(JAMMO_INSTRUMENT_TRACK(track),NULL);
+			//set to realtime
+			jammo_slider_track_set_recording (JAMMO_SLIDER_TRACK(static_slider_track), TRUE);
 
-		//set to realtime
-		jammo_instrument_track_set_realtime (JAMMO_INSTRUMENT_TRACK(track), TRUE);
+			//Change this slider_track to move slider_marker.
+			g_signal_connect (static_slider_track, "report-current-frequency", G_CALLBACK (update_marker),NULL);
+			printf("nauhoitus alkaa\n");
+		}
+
+
+		//Keyboard:
+		else {
+			//clean original
+			JammoTrack* track;
+			g_object_get(static_jammo_miditrack_view,"track",&track,NULL);
+			jammo_instrument_track_set_event_list(JAMMO_INSTRUMENT_TRACK(track),NULL);
+
+			//set to realtime
+			jammo_instrument_track_set_realtime (JAMMO_INSTRUMENT_TRACK(track), TRUE);
+		}
 
 		//Now all content in sequencer will play and instrument_track is recording
 		jammo_sequencer_play (JAMMO_SEQUENCER (static_sequencer));
@@ -265,31 +408,51 @@ void instrument_record_clicked (TangleButton *tanglebutton, gpointer user_data)
 static void instrument_gui_on_sequencer_stopped(JammoSequencer* sequencer, gpointer user_data) {
 	printf("instrument_gui_on_sequencer_stopped\n");
 
+	//Slider:
+	if (static_jammo_miditrack_view==NULL) {
+		//No need to copy events like in miditrack.
+
+		//If playback has just stopped, this is already FALSE, but it is not harm
+		jammo_slider_track_set_recording (JAMMO_SLIDER_TRACK(static_slider_track), FALSE);
+		//Change this slider_track to move slider_marker.
+		g_signal_connect (ad_hoc_slider_track, "report-current-frequency", G_CALLBACK (update_marker),NULL);
+	}
+
+
 	if (recording) {
 		recording = FALSE;
 
-		JammoTrack* track;
-		g_object_get(static_jammo_miditrack_view,"track",&track,NULL);
+		//Keyboard
+		if (static_jammo_miditrack_view){
+			JammoTrack* track;
+			g_object_get(static_jammo_miditrack_view,"track",&track,NULL);
 
-		//set to non-realtime (unless we cant use jammo_miditrack_view_add_event_list)
-		jammo_instrument_track_set_realtime (JAMMO_INSTRUMENT_TRACK(track), FALSE);
+			//set to non-realtime (unless we cant use jammo_miditrack_view_add_event_list)
+			jammo_instrument_track_set_realtime (JAMMO_INSTRUMENT_TRACK(track), FALSE);
 
-		//take all note-events from track
-		GList* list= jammo_instrument_track_get_event_list(JAMMO_INSTRUMENT_TRACK(track));
-		//Clean track
-		jammo_instrument_track_set_event_list(JAMMO_INSTRUMENT_TRACK(track),NULL);
-		//And put them back via miditrack_view
-		jammo_miditrack_view_add_event_list(static_jammo_miditrack_view,list);
-
-
-		//Start ad-hoc-sequencer
-		jammo_sequencer_play  (JAMMO_SEQUENCER (ad_hoc_sequencer));
+			//take all note-events from track
+			GList* list= jammo_instrument_track_get_event_list(JAMMO_INSTRUMENT_TRACK(track));
+			//Clean track
+			jammo_instrument_track_set_event_list(JAMMO_INSTRUMENT_TRACK(track),NULL);
+			//And put them back via miditrack_view
+			jammo_miditrack_view_add_event_list(static_jammo_miditrack_view,list);
+		}
 	}
 
+	//else = playback has just stopped
+
+
+	//Start ad-hoc-sequencer
+	jammo_sequencer_play  (JAMMO_SEQUENCER (ad_hoc_sequencer));
 }
 
-
-void instrument_gui_start (JammoMiditrackView* miditrack_view, JammoSequencer* parameter_sequencer, gboolean slider)
+/*
+Call this
+a) instrument_gui_start(JAMMO_MIDITRACK_VIEW(track->track_view),sequencer, NULL);
+or
+b) instrument_gui_start(NULL,sequencer, TANGLE_WIDGET(track->track_view));
+*/
+void instrument_gui_start (JammoMiditrackView* miditrack_view, JammoSequencer* parameter_sequencer, TangleWidget* slider_container)
 {
 	static_sequencer = parameter_sequencer;
 	g_signal_connect(static_sequencer, "stopped", G_CALLBACK(instrument_gui_on_sequencer_stopped), NULL);
@@ -304,13 +467,41 @@ void instrument_gui_start (JammoMiditrackView* miditrack_view, JammoSequencer* p
 	recording = FALSE;
 
 	ad_hoc_sequencer = jammo_sequencer_new ();
+	static_slider_track = NULL;
+	static_jammo_miditrack_view = NULL;
 
+	if (slider_container){
+		printf("starting slider\n");
+		clutter_actor_hide(jammo_get_actor_by_id("instrument-two-rows-keyboard")); //this will also hide go_editor-button
 
-	if (slider){ //TODO
-		clutter_actor_hide(jammo_get_actor_by_id("instrument-two-rows-keyboard"));
+		//This could be not-visible if midikeyboard has used
+		clutter_actor_show(jammo_get_actor_by_id("instrument-slider"));
+
+		static_slider_track = JAMMO_SLIDER_TRACK(g_object_get_data(G_OBJECT(slider_container),"track"));
+
+		//Make ad-hoc-track using same type
+		int type;
+		g_object_get(static_slider_track,"slider-type",&type,NULL);
+
+		ad_hoc_slider_track = jammo_slider_track_new (type);
+		jammo_slider_track_set_recording (ad_hoc_slider_track, TRUE);
+		jammo_sequencer_add_track (ad_hoc_sequencer, JAMMO_TRACK (ad_hoc_slider_track));
+		g_signal_connect (ad_hoc_slider_track, "report-current-frequency", G_CALLBACK (update_marker),NULL);
+
+		//Marker = cursor for slider frequency.
+		slider_marker =  jammo_get_actor_by_id("instrument-slider-marker");
+		//TODO: change texture to match selected type
+		//clutter_texture_new_from_file("/opt/jammo/slider_marker.png", NULL);
+		//After changing texture, tune anchor-point
+		clutter_actor_set_anchor_point(slider_marker,clutter_actor_get_width(slider_marker)/2,0.0);
 	}
 
 	else {
+		//This could be not-visible if slider has used
+		clutter_actor_show(jammo_get_actor_by_id("instrument-two-rows-keyboard"));
+
+		clutter_actor_hide(jammo_get_actor_by_id("instrument-slider"));
+
 		//Take miditrack_view safe to static variable
 		static_jammo_miditrack_view=miditrack_view;
 
