@@ -80,12 +80,15 @@ static JammoSample* get_centermost_sample_from_loop_roll(ClutterActor* wheel) {
 	gfloat y;
 	gfloat offset=0; //How much scroller is scrolled
 	gfloat height;    //Height of the scroller (we take this from color_box behind samples)
+	TangleWidget* container;
 
 	//Wheels first actor is scroller
 	ClutterActor* scroller = clutter_actor_get_parent(wheel);
 	if (TANGLE_IS_SCROLLING_ACTOR(scroller)){
+		container = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(scroller)));
 		offset = tangle_scrolling_actor_get_scrolling_offset_y(TANGLE_SCROLLING_ACTOR(scroller));
-		offset+=95; //height of the sample buttons. (we are looking sample in SECOND slot)
+		//we are looking sample in SECOND slot
+		offset+= clutter_actor_get_height(tangle_widget_get_nth_child(container,0)); //we suppose all have same height
 		//printf("offset = %f\n",offset);
 	}
 	else {
@@ -93,11 +96,8 @@ static JammoSample* get_centermost_sample_from_loop_roll(ClutterActor* wheel) {
 		return NULL;
 	}
 
-	TangleWidget* container = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(scroller)));
-
 	//Check height of the wheel
-	ClutterActor* color_box = tangle_widget_get_nth_child(container,0);
-	height = clutter_actor_get_height(color_box);
+	height = clutter_actor_get_height(CLUTTER_ACTOR(container));
 	//printf("height = %f\n",height);
 
 	//Check all sample button which one is closest to center (='offset')
@@ -182,7 +182,7 @@ void fullsequencer_loop_cannot_fail_clicked (TangleAction *action, GObject *sour
 }
 
 
-
+//There will be some logic
 void fullsequencer_loop_crazy_clicked (TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
 	GRand *random_gen = g_rand_new();
 
@@ -199,9 +199,6 @@ void fullsequencer_loop_crazy_clicked (TangleAction *action, GObject *source, co
 	g_rand_free(random_gen);
 }
 
-void fullsequencer_loop_handle_clicked (TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
-	printf("fullsequencer_loop_handle_clicked: not implemented yet\n");
-}
 
 //////////////////////////////////////
 
@@ -212,9 +209,6 @@ void sequencer_handle_dragging (TangleDroppableActor *droppable_actor, TangleDra
 
 
 void sequencer_loop_tune_wheels() {
-	//Disable scrolling, use only arrow-buttons
-	g_object_set(jammo_get_actor_by_id("fullsequencer-scroller"), "scrolling-threshold-y" , G_MAXFLOAT, "scrolling-threshold-x" , G_MAXFLOAT, NULL);
-
 	//Static
 	rhytmical_sample_looper  =  jammo_get_actor_by_id("fullsequencer-loop-rhytmical-samples");
 	melodical_sample_looper  =  jammo_get_actor_by_id("fullsequencer-loop-melodical-samples");
