@@ -18,7 +18,7 @@ and some metadata.
  * to add and remove TrackViews easily to sequencer
  */
 typedef struct {
-	ClutterActor *sequencer_track_view_container;
+	ClutterActor *control_panel;
 	ClutterActor *track_view;
 	ClutterActor *label;
 	ClutterActor *mute_button;
@@ -43,8 +43,8 @@ enum {
 };
 
 
-ClutterActor *sequencer_track_view_create_area_for_tracks(JammoSequencer* seq);
-ClutterActor* sequencer_track_view_add_with_type(gchar* track_view_type, guint64 duration);
+void sequencer_track_view_create_area_for_tracks(JammoSequencer* seq, guint n_slots, guint64 total_duration, gfloat slot_width, gfloat height);
+ClutterActor* sequencer_track_view_add_with_type(gchar* track_view_type);
 
 //void sequencer_sequencer_track_view_remove_last();
 bool save_sequencer_state();
