@@ -51,11 +51,6 @@ struct jammo_collaboration_game_user_data {
 	ClutterActor * view;
 };
 
-// TODO add actual code to callback
-static void jammo_collaboration_game_group_callback(GObject * game, gint type) {
-	printf("jammo-collaboration-game: group_callback, type %d\n", type);
-}
-
 static void jammo_collaboration_game_get_property(GObject* object, guint prop_id, GValue* value, GParamSpec* pspec) {
 	JammoCollaborationGame * collaboration_game;
 	
@@ -179,6 +174,14 @@ int jammo_collaboration_game_create_song_file(JammoCollaborationGame * collabora
 	return (JAMMO_COLLABORATION_GAME_GET_CLASS (collaboration_game)->create_song_file(collaboration_game));
 }
 
+void jammo_collaboration_game_game_callback(GObject * game, int type) {
+	JammoCollaborationGame * collaboration_game = JAMMO_COLLABORATION_GAME(game);
+	
+	// TODO common operations for all games
+
+	JAMMO_COLLABORATION_GAME_GET_CLASS (collaboration_game)->game_callback(game, type);
+}
+
 static GObject* jammo_collaboration_game_constructor(GType type, guint n_properties, GObjectConstructParam* properties) {
 	GObject* object;
 	JammoCollaborationGame * collaboration_game;
@@ -253,7 +256,7 @@ static void jammo_collaboration_game_class_init(JammoCollaborationGameClass* col
 
 	// group callback for gems
 	// this is used in passing group related information to game, e.g. player joined game
-	gems_set_callback_group_control(jammo_collaboration_game_group_callback);
+	gems_set_callback_game(jammo_collaboration_game_game_callback);
 
 	g_object_class_install_property(gobject_class, PROP_BACKING_TRACK_LOCATION,
 	                                g_param_spec_string("backing-track-location",
