@@ -24,7 +24,15 @@ enum {
 	PROP_LINE_EVERY_NTH_SLOT,
 	PROP_DISABLED_SLOTS_BEGIN,
 	PROP_DISABLED_SLOTS_END,
-	PROP_EDITING_ENABLED
+	PROP_EDITING_ENABLED,
+	PROP_SAMPLE_COLOR,
+	PROP_HILIGHT_COLOR,
+	PROP_SAMPLE_TYPE
+};
+
+enum {
+	DROPPING_INCOMPATIBLE_SAMPLE,
+	LAST_SIGNAL
 };
 
 struct _JammoTrackViewPrivate {
@@ -36,7 +44,9 @@ struct _JammoTrackViewPrivate {
 	guint line_every_nth_slot;
 	guint disabled_slots_begin;
 	guint disabled_slots_end;
+	ClutterColor sample_color;
 	ClutterColor hilight_color;
+	JammoSampleType sample_type;
 	
 	gfloat real_slot_width;
 	gfloat hilighted_start;
@@ -46,9 +56,20 @@ struct _JammoTrackViewPrivate {
 	guint editing_enabled : 1;
 };
 
+static guint signals[LAST_SIGNAL];
+
+static const ClutterColor default_sample_color = { 0, 0, 0, 0 };
+static const ClutterColor default_hilight_color = { 255, 255, 255, 100 };
+
 static void on_actor_added(TangleWidget* widget, ClutterActor* actor, gpointer user_data);
 static void on_actor_removed(TangleWidget* widget, ClutterActor* actor, gpointer user_data);
 static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* sample_button, guint slot_index);
+static void marshal_BOOLEAN__OBJECT (GClosure     *closure,
+                                GValue       *return_value G_GNUC_UNUSED,
+                                guint         n_param_values,
+                                const GValue *param_values,
+                                gpointer      invocation_hint G_GNUC_UNUSED,
+                                gpointer      marshal_data);
 
 ClutterActor* jammo_track_view_new(JammoEditingTrack* track, guint n_slots, guint64 slot_duration, gfloat slot_width, gfloat slot_height) {
 
@@ -124,12 +145,24 @@ void jammo_track_view_set_editing_enabled(JammoTrackView* track_view, gboolean e
 	}
 }
 
+JammoSampleType jammo_track_view_get_sample_type(JammoTrackView* track_view) {
+
+	return track_view->priv->sample_type;
+}
+
+void jammo_track_view_set_sample_type(JammoTrackView* track_view, JammoSampleType sample_type) {
+	if (track_view->priv->sample_type != sample_type) {
+		track_view->priv->sample_type = sample_type;
+		g_object_notify(G_OBJECT(track_view), "sample-type");
+	}
+}
+
 static gboolean jammo_track_view_is_dragging_droppable(TangleDroppableActor* droppable_actor, TangleDragging* dragging) {
 	gboolean droppable = FALSE;
 	JammoTrackView* track_view;
 	gfloat center_x, center_y;
-	guint slot;
-	guint n_slots;
+	guint slot = 0;
+	guint n_slots = 0;
 	gfloat start;
 	
 	track_view = JAMMO_TRACK_VIEW(droppable_actor);
@@ -144,16 +177,17 @@ static gboolean jammo_track_view_is_dragging_droppable(TangleDroppableActor* dro
 		    slot + n_slots - 1 < track_view->priv->n_slots - track_view->priv->disabled_slots_end &&
 		    !check_if_overlapping(track_view, CLUTTER_ACTOR(dragging->draggable_actor), slot)) {
 			droppable = TRUE;
-			start = slot * track_view->priv->real_slot_width;
-			if (start != track_view->priv->hilighted_start) {
-				track_view->priv->hilighted_start = start;
-				track_view->priv->hilighted_end = start + n_slots * track_view->priv->real_slot_width;
-				clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
-			}
 		}
 	}
 	
-	if (!droppable && track_view->priv->hilighted_start != track_view->priv->hilighted_end) {
+	if (droppable) {
+		start = slot * track_view->priv->real_slot_width;
+		if (start != track_view->priv->hilighted_start) {
+			track_view->priv->hilighted_start = start;
+			track_view->priv->hilighted_end = start + n_slots * track_view->priv->real_slot_width;
+			clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
+		}
+	} else if (track_view->priv->hilighted_start != track_view->priv->hilighted_end) {
 		track_view->priv->hilighted_start = track_view->priv->hilighted_end = 0.0;
 		clutter_actor_queue_redraw(CLUTTER_ACTOR(track_view));
 	}
@@ -232,9 +266,20 @@ static gboolean jammo_track_view_handle_dropping(TangleDroppableActor* droppable
 
 	if (accepted && clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), dragging->x, dragging->y, &x, &y) &&
 	    clutter_actor_transform_stage_point(CLUTTER_ACTOR(track_view), dragging->center_x, dragging->center_y, &center_x, &center_y)) {
+		if (track_view->priv->sample_type != JAMMO_SAMPLE_UNTYPED &&
+		     jammo_sample_button_get_sample_type(JAMMO_SAMPLE_BUTTON(dragging->draggable_actor)) != track_view->priv->sample_type &&
+		     jammo_sample_button_get_sample_type(JAMMO_SAMPLE_BUTTON(dragging->draggable_actor)) != JAMMO_SAMPLE_UNTYPED) {
+		     	accepted = FALSE;
+			g_signal_emit(track_view, signals[DROPPING_INCOMPATIBLE_SAMPLE], 0, dragging->draggable_actor, &accepted);
+		}
+	}
+
+	if (accepted) {
 		sample_button = JAMMO_SAMPLE_BUTTON(dragging->draggable_actor);
 		copy_of_sample_button = jammo_sample_button_new_from_existing(sample_button);
 
+		jammo_sample_button_set_background_color(JAMMO_SAMPLE_BUTTON(copy_of_sample_button), &track_view->priv->sample_color);
+
 		n_slots = (guint)(clutter_actor_get_width(CLUTTER_ACTOR(dragging->draggable_actor)) / track_view->priv->real_slot_width + 0.5);
 		center_x -= (n_slots - 1) * track_view->priv->real_slot_width / 2;
 		slot = center_x / track_view->priv->real_slot_width;
@@ -346,6 +391,15 @@ static void jammo_track_view_set_property(GObject* object, guint prop_id, const
 		case PROP_EDITING_ENABLED:
 			jammo_track_view_set_editing_enabled(track_view, g_value_get_boolean(value));
 			break;
+		case PROP_SAMPLE_COLOR:
+			track_view->priv->sample_color = *clutter_value_get_color(value);
+			break;
+		case PROP_HILIGHT_COLOR:
+			track_view->priv->hilight_color = *clutter_value_get_color(value);
+			break;
+		case PROP_SAMPLE_TYPE:
+			track_view->priv->sample_type = g_value_get_enum(value);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 			break;
@@ -385,6 +439,15 @@ static void jammo_track_view_get_property(GObject* object, guint prop_id, GValue
 		case PROP_EDITING_ENABLED:
 			g_value_set_boolean(value, track_view->priv->editing_enabled);
 			break;
+		case PROP_SAMPLE_COLOR:
+			clutter_value_set_color(value, &track_view->priv->sample_color);
+			break;
+		case PROP_HILIGHT_COLOR:
+			clutter_value_set_color(value, &track_view->priv->hilight_color);
+			break;
+		case PROP_SAMPLE_TYPE:
+			g_value_set_enum(value, track_view->priv->sample_type);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		        break;
@@ -522,6 +585,59 @@ static void jammo_track_view_class_init(JammoTrackViewClass* track_view_class) {
 	                                "Whether the track can be edited or not",
 	                                TRUE,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * JammoTrackView:sample-color:
+	 *
+	 * A color that is used as a background color of dropped samples.
+	 */
+	g_object_class_install_property(gobject_class, PROP_SAMPLE_COLOR,
+	                                clutter_param_spec_color("sample-color",
+	                                "Sample color",
+	                                "A color that is used as a background color of dropped samples",
+	                                &default_sample_color,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * JammoTrackView:hilight-color:
+	 *
+	 * A color that is used to hilight selected slots when dragging samples.
+	 */
+	g_object_class_install_property(gobject_class, PROP_HILIGHT_COLOR,
+	                                clutter_param_spec_color("hilight-color",
+	                                "Hilight color",
+	                                "A color that is used to hilight selected slots when dragging samples",
+	                                &default_sample_color,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * JammoTrackView:sample-type:
+	 *
+	 * The type of samples that are allowed to be dropped into the track. If a #JammoTrackView or a #JammoSampleButton has :sample-type
+	 * set to JAMMO_SAMPLE_UNTYPE dropping is always allowed.
+	 */
+	g_object_class_install_property(gobject_class, PROP_SAMPLE_TYPE,
+	                                g_param_spec_enum("sample-type",
+	                                "Sample type",
+	                                "The type of samples that are allowed to be dropped into the track",
+	                                JAMMO_TYPE_SAMPLE_TYPE, JAMMO_SAMPLE_UNTYPED,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+
+	/**
+	 * JammoTrackView:dropping-incompatible-sample:
+	 * @sample_button: the #JammoSampleButton that is being dropped and is incompatible in terms of #JammoSampleType.
+	 *
+	 * The :dropping-incompatible-sample signal is being emitted when an user tries to drag (and drop) a sample button
+	 * that has a different #JammoSampleType than the track (see :sample-type). If either has JAMMO_SAMPLE_UNTYPED,
+	 * the compatability check is not being performed.
+	 *
+	 * The signal handler should return FALSE, if the dropping should fail (in that case, also other handlers are
+	 * being called), or TRUE, if the dropping should be possible (no other handlers are being called after that).
+	 */
+	signals[DROPPING_INCOMPATIBLE_SAMPLE] = g_signal_new("dropping-incompatible-sample", G_TYPE_FROM_CLASS(gobject_class),
+	                                		     G_SIGNAL_RUN_LAST, 0,
+							     g_signal_accumulator_true_handled, NULL,
+							     marshal_BOOLEAN__OBJECT,
+							     G_TYPE_BOOLEAN, 1,
+							     JAMMO_TYPE_SAMPLE_BUTTON);
+
 
 	g_type_class_add_private (gobject_class, sizeof (JammoTrackViewPrivate));
 }
@@ -530,10 +646,8 @@ static void jammo_track_view_init(JammoTrackView* track_view) {
 	track_view->priv = G_TYPE_INSTANCE_GET_PRIVATE(track_view, JAMMO_TYPE_TRACK_VIEW, JammoTrackViewPrivate);
 	track_view->priv->editing_enabled = TRUE;
 
-	track_view->priv->hilight_color.red = 255;
-	track_view->priv->hilight_color.green = 255;
-	track_view->priv->hilight_color.blue = 255;
-	track_view->priv->hilight_color.alpha = 100;
+	track_view->priv->sample_color = default_sample_color;
+	track_view->priv->hilight_color = default_hilight_color;
 
 	track_view->priv->hilighted_start = track_view->priv->hilighted_end = -1.0;
 }
@@ -604,3 +718,53 @@ static gboolean check_if_overlapping(JammoTrackView* track_view, ClutterActor* s
 	
 	return actor != NULL;
 }
+
+/* Generated marshaler */
+
+#ifdef G_ENABLE_DEBUG
+#define g_marshal_value_peek_object(v)   g_value_get_object (v)
+#else /* !G_ENABLE_DEBUG */
+/* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
+ *          Do not access GValues directly in your code. Instead, use the
+ *          g_value_get_*() functions
+ */
+#define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
+#endif /* !G_ENABLE_DEBUG */
+
+void marshal_BOOLEAN__OBJECT (GClosure     *closure,
+                              GValue       *return_value G_GNUC_UNUSED,
+                              guint         n_param_values,
+                              const GValue *param_values,
+                              gpointer      invocation_hint G_GNUC_UNUSED,
+                              gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__OBJECT) (gpointer     data1,
+                                                    gpointer     arg_1,
+                                                    gpointer     data2);
+  register GMarshalFunc_BOOLEAN__OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_object (param_values + 1),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
