@@ -36,7 +36,6 @@ typedef struct _JammoTrackViewClass {
 GType jammo_track_view_get_type(void) G_GNUC_CONST;
 
 ClutterActor* jammo_track_view_new(JammoEditingTrack* track, guint n_slots, guint64 slot_duration, gfloat slot_width, gfloat slot_height);
-ClutterActor* jammo_track_view_new_peer(JammoEditingTrack* track, guint n_slots, guint64 slot_duration, gfloat slot_width, gfloat slot_height);
 
 void jammo_track_view_remove_jammo_sample_button_from_slot(JammoTrackView* track_view, guint asked_slot);
 void jammo_track_view_add_jammo_sample_button(JammoTrackView* track_view, JammoSampleButton* sample_button, guint slot);
@@ -45,4 +44,7 @@ guint jammo_track_view_get_sample_button_slot(JammoTrackView* track_view, JammoS
 gboolean jammo_track_view_get_editing_enabled(JammoTrackView* track_view);
 void jammo_track_view_set_editing_enabled(JammoTrackView* track_view, gboolean editing_enabled);
 
+JammoSampleType jammo_track_view_get_sample_type(JammoTrackView* track_view);
+void jammo_track_view_set_sample_type(JammoTrackView* track_view, JammoSampleType sample_type);
+
 #endif
