@@ -29,7 +29,8 @@ static void empty_callback_remove_loop_from_slot(GObject * game, guint user_id,
 static void empty_callback_loop_sync(GObject * game, guint user_id, GList * list){}
 static void empty_callback_midi_list(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list){}
 static void empty_callback_slider_event_list(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list){}
-static void empty_callback_group_control(GObject * game, gint type){}
+static void empty_callback_group_control(gint type){}
+static void empty_callback_game(GObject * game, gint type){}
 
 // A "singleton" get
 gems_components* gems_get_data()
@@ -45,9 +46,11 @@ gems_components* gems_get_data()
 		gems_data->service_jammo = NULL;
 		gems_data->service_profile = NULL;
 		gems_data->service_group = NULL;
+		gems_data->service_control = NULL;
 		gems_data->initialized_connections = NULL;
 		gems_data->connection_errorlist = NULL;
 		gems_data->service_collaboration = NULL;
+		gems_data->teacher_connection = NULL;
 	}
 	return gems_data;
 }
@@ -103,11 +106,20 @@ gems_service_collaboration* gems_get_service_collaboration()
 	return gems_get_data()->service_collaboration;
 }
 
+gems_service_control* gems_get_service_control()
+{
+	return gems_get_data()->service_control;
+}
+
 GList* gems_get_errorlist()
 {
 	return gems_get_data()->connection_errorlist;
 }
 
+gems_teacher_connection* gems_get_teacher_connection()
+{
+	return gems_get_data()->teacher_connection;
+}
 
 void gems_free_data()
 {
@@ -152,6 +164,9 @@ void jammo_gems_init(int argc, char* argv[])
 	// Initialize Control service
 	components->service_control = gems_service_control_init();
 	
+	// Initialize teacher connection
+	components->teacher_connection = gems_teacher_connection_init();
+	
 	// Define function pointers for notify and new connection functions
 	void (*notify)(short, const char*,void*) = NULL;
 	notify = &gems_ph_callback_notify;
@@ -174,6 +189,7 @@ void jammo_gems_init(int argc, char* argv[])
 		components->service_profile->enabled = FALSE;
 		components->service_group->enabled = FALSE;
 		components->service_collaboration->enabled = FALSE;
+		components->service_control->enabled = FALSE;
 		//gems_service_jammo_cleanup(components->service_jammo);
 		
 		//gems_communication_cleanup(components->communication);
@@ -195,6 +211,7 @@ void jammo_gems_init(int argc, char* argv[])
 	chum_callback_midi_list=empty_callback_midi_list;
 	chum_callback_slider_event_list=empty_callback_slider_event_list;
 	chum_callback_group_control=empty_callback_group_control;
+	chum_callback_game=empty_callback_game;
 
 	logmsg = g_strdup_printf("jammo_gems_init: done");
 	cem_add_to_log(logmsg,LOG_DEBUG);
@@ -240,6 +257,8 @@ void jammo_gems_cleanup()
 	if(components->service_group->enabled == TRUE) ph_c_unregister_service(components->ph,COLLABORATION_SERVICE_NAME);
 	gems_service_collaboration_cleanup();
 	
+	gems_clear_teacher_connection();
+	
 	// delete peerhood
 	if(components->ph != NULL)
 	{
@@ -373,6 +392,8 @@ void gems_ph_callback_notify(short aEvent, const char* aAddress,void* aData)
 void gems_ph_callback_newconnection(const unsigned short aPort, MAbstractConnection* aConnection, int aConnectionId, void* aData)
 {
 	gems_components* components = NULL;
+	gems_message* err = NULL;
+	gems_message* message = NULL;
 	gint isconnected = 0;
 	gchar* logmsg = NULL;
 	gchar* logerror = NULL;
@@ -385,34 +406,154 @@ void gems_ph_callback_newconnection(const unsigned short aPort, MAbstractConnect
 
 	if(components != NULL)
 	{
-		// If connecting to wrong port
-		if(aPort != components->service_jammo->port)
+		// Connecting to JamMo
+		if(aPort == components->service_jammo->port)
 		{
-			logmsg = g_strdup_printf("gems_ph_callback_newconnection: connecting to wrong port (connecting %d, jammo service %d)", aPort,
-				components->service_jammo->port);
+			if(components->service_jammo->enabled == FALSE)
+			{
+				err = gems_create_error_message(ERROR_SERVICE_NOT_RUNNING_TYPE, JAMMO_SERVICE_ID);
+					
+				if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),err) != err->length)
+				{
+					logerror = g_strdup_printf("gems_ph_callback_newconnection: cannot write ERROR_SERVICE_NOT_RUNNING (%s)",JAMMO_SERVICE_NAME);
+					cem_add_to_log(logerror,LOG_ERROR);
+					g_free(logerror);
+				}
+				else
+				{
+					logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_SERVICE_NOT_RUNNING (%s) to %s",JAMMO_SERVICE_NAME, ph_c_connection_get_remote_address(aConnection));
+					cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+					g_free(logmsg);
+				}
+			
+				gems_clear_message(err);
+				ph_c_connection_delete(aConnection);
+				return;
+			}
+		}
+		
+		// Connecting to Collaboration (game connection)
+		else if(aPort == components->service_collaboration->port)
+		{
+			if(components->service_collaboration->enabled == FALSE)
+			{
+				err = gems_create_error_message(ERROR_SERVICE_NOT_RUNNING_TYPE, COLLABORATION_SERVICE_ID);
+					
+				if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),err) != err->length)
+				{
+					logerror = g_strdup_printf("gems_ph_callback_newconnection: cannot write ERROR_SERVICE_NOT_RUNNING (%s)", COLLABORATION_SERVICE_NAME);
+					cem_add_to_log(logerror,LOG_ERROR);
+					g_free(logerror);
+				}
+				else
+				{
+					logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_SERVICE_NOT_RUNNING (%s) to %s",COLLABORATION_SERVICE_NAME, ph_c_connection_get_remote_address(aConnection));
+					cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+					g_free(logmsg);
+				}
+			
+				gems_clear_message(err);
+				ph_c_connection_delete(aConnection);
+				return;
+			}
+		}
+			
+		// Connecting to control (teacher)
+		else if(aPort == components->service_control->port)
+		{
+			if(components->service_control->enabled == FALSE)
+			{
+				err = gems_create_error_message(ERROR_SERVICE_NOT_RUNNING_TYPE, CONTROL_SERVICE_ID);
+					
+				if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),err) != err->length)
+				{
+					logerror = g_strdup_printf("gems_ph_callback_newconnection: cannot write ERROR_SERVICE_NOT_RUNNING (%s)", CONTROL_SERVICE_NAME);
+					cem_add_to_log(logerror,LOG_ERROR);
+					g_free(logerror);
+				}
+				else
+				{
+					logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_SERVICE_NOT_RUNNING (%s) to %s",CONTROL_SERVICE_NAME, ph_c_connection_get_remote_address(aConnection));
+					cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+					g_free(logmsg);
+				}
+			
+				gems_clear_message(err);
+				ph_c_connection_delete(aConnection);
+				return;
+			}
+			else
+			{
+				if(components->teacher_connection != NULL)
+				{
+					if(components->teacher_connection->connection == NULL)
+					{
+						components->teacher_connection->connection = aConnection;
+						logmsg = g_strdup_printf("gems_ph_callback_newconnection: TEACHER CONNECTED from %s", ph_c_connection_get_remote_address(aConnection));
+						cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+						g_free(logmsg);
+					}
+					// Connection was established before
+					else
+					{
+						err = gems_create_error_message(ERROR_CONNECTION_REJECTED_TYPE, CONTROL_SERVICE_ID);
+				
+						if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),err) != err->length)
+						{
+							logerror = g_strdup_printf("gems_ph_callback_newconnection: cannot write ERROR_CONNECTION_REJECTED (%s)", CONTROL_SERVICE_NAME);
+							cem_add_to_log(logerror,LOG_ERROR);
+							g_free(logerror);
+						}
+						else
+						{
+							logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_CONNECTION_REJECTED (%s) to TEACHER %s",CONTROL_SERVICE_NAME, ph_c_connection_get_remote_address(aConnection));
+							cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+							g_free(logmsg);
+						}
+						
+						gems_clear_message(err);
+						
+						logmsg = g_strdup_printf("gems_ph_callback_newconnection: TEACHER CONNECTED from %s, TEACHER is already connected!", ph_c_connection_get_remote_address(aConnection));
+						cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+						g_free(logmsg);
+						
+						ph_c_connection_delete(aConnection);
+					}
+					
+				}
+				return;
+			}
+		}
+		
+		// Invalid port
+		else
+		{
+			logmsg = g_strdup_printf("gems_ph_callback_newconnection: connecting to wrong port (connecting port %d)", aPort);
 			cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
 			g_free(logmsg);
-		
-			gems_message* message = gems_create_error_message(ERROR_CONNECTION_REJECTED_TYPE,JAMMO_SERVICE_ID);
-
-			if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),message) != message->length)
-			{				
-				logerror = g_strdup_printf("gems_ph_callback_newconnection: Cannot write error ERROR_CONNECTION_REJECTED");
+			
+			err = gems_create_error_message(ERROR_CONNECTION_REJECTED_TYPE, 0);
+				
+			if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),err) != err->length)
+			{
+				logerror = g_strdup_printf("gems_ph_callback_newconnection: cannot write ERROR_CONNECTION_REJECTED");
 				cem_add_to_log(logerror,LOG_ERROR);
 				g_free(logerror);
 			}
 			else
 			{
-				logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_ALREADY_CONNECTED message data to %s", ph_c_connection_get_remote_address(aConnection));
+				logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_CONNECTION_REJECTED to %s", ph_c_connection_get_remote_address(aConnection));
 				cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
 				g_free(logmsg);
 			}
 			
-			gems_clear_message(message);
+			gems_clear_message(err);
 			ph_c_connection_delete(aConnection);
+			return;
 		}
 		
-		else if((isconnected = gems_communication_already_connected(ph_c_connection_get_device_checksum(aConnection))) == NOT_IN_LIST)
+		// Not in any list - new connection
+		if((isconnected = gems_communication_already_connected(ph_c_connection_get_device_checksum(aConnection))) == NOT_IN_LIST)
 		{
 			logmsg = g_strdup_printf("gems_ph_callback_newconnection: Adding a connection with cid %d address %s..", aConnectionId, ph_c_connection_get_remote_address(aConnection));
 			cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
@@ -437,7 +578,7 @@ void gems_ph_callback_newconnection(const unsigned short aPort, MAbstractConnect
 			cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
 			g_free(logmsg);
 			
-			gems_message* message = gems_create_error_message(ERROR_CONNECTION_REJECTED_TYPE,JAMMO_SERVICE_ID);
+			message = gems_create_error_message(ERROR_CONNECTION_REJECTED_TYPE,JAMMO_SERVICE_ID);
 			
 			switch(gems_communication_write_data(ph_c_connection_get_fd(aConnection),message))
 			{
@@ -452,7 +593,7 @@ void gems_ph_callback_newconnection(const unsigned short aPort, MAbstractConnect
 					g_free(logerror);
 					break;
 				default:
-					logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_ALREADY_CONNECTED message data to %s", ph_c_connection_get_remote_address(aConnection));
+					logmsg = g_strdup_printf("gems_ph_callback_newconnection: Wrote ERROR_CONNECTION_REJECTED message data to %s", ph_c_connection_get_remote_address(aConnection));
 					cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
 					g_free(logmsg);
 					break;
@@ -464,32 +605,98 @@ void gems_ph_callback_newconnection(const unsigned short aPort, MAbstractConnect
 		
 		else // Already in some list
 		{
-			// Connected AND group active - might try to establish collaboration connection (for games)
-			if((isconnected == IN_CONNECTED_LIST) && (gems_group_active() == TRUE)) // TODO add checking of collaboration service port and connection port
+			// Connected AND connecting to collaboration service AND group active - might try to establish collaboration connection (for games)
+			if((isconnected == IN_CONNECTED_LIST))
 			{
-				gems_connection* element = gems_communication_get_connection(gems_get_data()->communication->connections,ph_c_connection_get_device_checksum(aConnection));
-				
-				if(element != NULL) // Found
+				// Collaboration service was checked earlier
+				if(aPort == components->service_collaboration->port)
 				{
-					if(element->profile != NULL) // Profile requested
+					// Is group active?
+					if(gems_group_active() == TRUE)
 					{
-						if(gems_group_is_in_group(element->profile->id) == TRUE) // In our group
+						// Get connection for this checksum
+						gems_connection* element = gems_communication_get_connection(gems_get_data()->communication->connections,ph_c_connection_get_device_checksum(aConnection));
+		
+						if(element != NULL) // Found - should be always found
 						{
-							element->game_connection = aConnection; // Add as game connection
-							logmsg = g_strdup_printf("gems_ph_callback_newconnection: Added Game connection for Collaboration");
-							cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
-							g_free(logmsg);
-							return;
+							if(element->profile != NULL) // Profile requested
+							{
+								if(gems_group_is_in_group(element->profile->id) == TRUE) // In our group
+								{
+									element->game_connection = aConnection; // Add as game connection
+									logmsg = g_strdup_printf("gems_ph_callback_newconnection: Added Game connection for Collaboration");
+									cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+									g_free(logmsg);
+									return;
+								}
+								// Not in our group - disconnect
+								else
+								{
+									logerror = g_strdup_printf("gems_ph_callback_newconnection: Connection from %s/%d to %s who is not in the same group, disconnecting.", 
+									ph_c_connection_get_remote_address(aConnection), ph_c_connection_get_device_checksum(aConnection), COLLABORATION_SERVICE_NAME);
+									cem_add_to_log(logerror,LOG_ERROR);
+									g_free(logerror);
+									
+									err = gems_create_error_message(ERROR_NOT_IN_GROUP_TYPE,COLLABORATION_SERVICE_ID);
+		
+									if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),err) != err->length)
+									{
+										logerror = g_strdup_printf("gems_ph_callback_newconnection: error sending ERROR_NOT_IN_GROUP to %s/%d",
+											ph_c_connection_get_remote_address(aConnection), ph_c_connection_get_device_checksum(aConnection));
+										cem_add_to_log(logerror,LOG_ERROR);
+										g_free(logerror);
+									}
+									gems_clear_message(err);
+						
+									ph_c_connection_delete(aConnection);
+									return;
+								}
+							}
+							else
+							{
+								// TODO error - no profile
+								logerror = g_strdup_printf("gems_ph_callback_newconnection: Connection from %s/%d to %s - no profile retrieved yet, disconnecting.", 
+								ph_c_connection_get_remote_address(aConnection), ph_c_connection_get_device_checksum(aConnection), COLLABORATION_SERVICE_NAME);
+								cem_add_to_log(logerror,LOG_ERROR);
+								g_free(logerror);
+							
+								ph_c_connection_delete(aConnection);
+								return;
+							}
 						}
 					}
+					else
+					{
+						//error - group not active
+						logerror = g_strdup_printf("gems_ph_callback_newconnection: Connection from %s/%d to %s when group was not active.", 
+						ph_c_connection_get_remote_address(aConnection), ph_c_connection_get_device_checksum(aConnection), COLLABORATION_SERVICE_NAME);
+						cem_add_to_log(logerror,LOG_ERROR);
+						g_free(logerror);
+					
+						err = gems_create_error_message(ERROR_GROUP_NOT_ACTIVE_TYPE,COLLABORATION_SERVICE_ID);
+
+						if(gems_communication_write_data(ph_c_connection_get_fd(aConnection),err) != err->length)
+						{
+							logerror = g_strdup_printf("gems_ph_callback_newconnection: error sending ERROR_GROUP_NOT_ACTIVE to %s/%d",
+								ph_c_connection_get_remote_address(aConnection), ph_c_connection_get_device_checksum(aConnection));
+							cem_add_to_log(logerror,LOG_ERROR);
+							g_free(logerror);
+						}
+
+						gems_clear_message(err);
+				
+						ph_c_connection_delete(aConnection);
+						return;
+					}
 				}
 			}
 			
+			// Otherwise already connected, send error
 			logmsg = g_strdup_printf("gems_ph_callback_newconnection: device is already connected");
 			cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
 			g_free(logmsg);
 			
-			gems_message* message = gems_create_error_message(ERROR_ALREADY_CONNECTED_TYPE,JAMMO_SERVICE_ID);
+			message = gems_create_error_message(ERROR_ALREADY_CONNECTED_TYPE,JAMMO_SERVICE_ID);
 		
 			switch(gems_communication_write_data(ph_c_connection_get_fd(aConnection),message))
 			{
@@ -764,9 +971,12 @@ void gems_set_callback_midi_list(void (*func_ptr)(GObject * game, guint user_id,
 void gems_set_callback_slider_event_list(void (*func_ptr)(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list) ) {
 	chum_callback_slider_event_list=func_ptr;
 }
-void gems_set_callback_group_control(void (*func_ptr)(GObject * game, gint type) ) {
+void gems_set_callback_group_control(void (*func_ptr)(gint type) ) {
 	chum_callback_group_control=func_ptr;
 }
+void gems_set_callback_game(void (*func_ptr)(GObject * game, gint type) ) {
+	chum_callback_game=func_ptr;
+}
 
 void gems_debugging_show_jammos()
 {
@@ -817,7 +1027,9 @@ void gems_enable_network_timeout_functions()
 	g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE,200,(GSourceFunc)gems_service_jammo_process_connections, NULL, NULL);
 	g_timeout_add_full(G_PRIORITY_LOW,200,(GSourceFunc)gems_communication_search_jammos, NULL, NULL);
 	g_timeout_add_full(G_PRIORITY_LOW,300,(GSourceFunc)gems_communication_retrieve_profiles, NULL, NULL);
-	g_timeout_add_full(G_PRIORITY_DEFAULT,100,(GSourceFunc)gems_communication_process_connections, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_DEFAULT,200,(GSourceFunc)gems_communication_process_connecting, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_DEFAULT,100,(GSourceFunc)gems_communication_process_connected, NULL, NULL);
+	g_timeout_add_full(G_PRIORITY_LOW,500,(GSourceFunc)gems_communication_process_rejected, NULL, NULL);
 	g_timeout_add_full(G_PRIORITY_LOW,200,(GSourceFunc)gems_communication_request_groups, NULL, NULL);
 	g_timeout_add_full(G_PRIORITY_LOW,350,(GSourceFunc)gems_communication_sanitize_grouplist, NULL, NULL);
 }
@@ -885,6 +1097,9 @@ void gems_register_services()
 		components->service_collaboration->enabled = FALSE;
 		logmsg = g_strdup_printf("gems_networking_register_services: cannot register %s service.",COLLABORATION_SERVICE_NAME);
 	}
+	cem_add_to_log(logmsg,LOG_NETWORK_DEBUG);
+	g_free(logmsg);
+	
 	// Create Control service
 	if((serviceport = ph_c_register_service(components->ph,CONTROL_SERVICE_NAME,"application:jammo,method:control")) != 0)
 	{
