@@ -44,6 +44,8 @@ gems_service_jammo* gems_get_service_jammo();
 gems_service_profile* gems_get_service_profile();
 gems_service_group* gems_get_service_group();
 gems_service_collaboration* gems_get_service_collaboration();
+gems_service_control* gems_get_service_control();
+gems_teacher_connection* gems_get_teacher_connection();
 
 GList* gems_get_errorlist();
 
@@ -84,14 +86,16 @@ void (*chum_callback_add_loop_to_slot)(GObject * game, guint user_id, guint id,
 void (*chum_callback_remove_loop_from_slot)(GObject * game, guint user_id, guint slot);
 void (*chum_callback_loop_sync)(GObject * game, guint user_id, GList * list);
 void (*chum_callback_midi_list)(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list);
-void (*chum_callback_group_control)(GObject * game, gint type);
+void (*chum_callback_group_control)(gint type);
+void (*chum_callback_game)(GObject * game, gint type);
 void (*chum_callback_slider_event_list)(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list);
 void gems_set_callback_add_loop(void (*func_ptr)(GObject * game, guint user_id, guint id, guint slot) );
 void gems_set_callback_remove_loop(void (*func_ptr)(GObject * game, guint user_id, guint slot) );
 void gems_set_callback_loop_sync(void (*func_ptr)(GObject * game, guint user_id, GList * list) );
 void gems_set_callback_midi_list(void (*func_ptr)(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list) );
 void gems_set_callback_slider_event_list(void (*func_ptr)(GObject * game, guint user_id, gint16 instrument_id, GemsEventListOperation operation, GList * list) );
-void gems_set_callback_group_control(void (*func_ptr)(GObject * game, gint type) );
+void gems_set_callback_group_control(void (*func_ptr)(gint type) );
+void gems_set_callback_game(void (*func_ptr)(GObject * game, gint type) );
 
 void gems_init_peerhood_callback();
 void gems_enable_network_timeout_functions();
