@@ -20,6 +20,30 @@ typedef enum
 	GEMS_EVENTLIST_REPLACE
 } GemsEventListOperation;
 
+typedef enum
+{
+	GEMS_GROUP_CONTROL_NOT_IN_GROUP,
+	GEMS_GROUP_CONTROL_GROUP_NOT_ACTIVE,
+	GEMS_GROUP_CONTROL_GROUP_FULL,
+	GEMS_GROUP_CONTROL_ALREADY_IN_GROUP,
+	GEMS_GROUP_CONTROL_GROUP_LOCKED,
+	GEMS_GROUP_CONTROL_GROUP_ADD_ERROR,
+	GEMS_GROUP_CONTROL_NOT_GROUP_OWNER
+} GemsGroupControlCallbackType;
+
+typedef enum
+{
+	GEMS_GAME_JOINED_PAIR_COMPOSITION
+} GemsGameCallbackType;
+
+#define ERROR_GROUP_NOT_ACTIVE_TYPE 10008
+#define ERROR_INVALID_GROUP_ID_TYPE 10009
+#define ERROR_GROUP_FULL_TYPE 10010
+#define ERROR_ALREADY_IN_GROUP_TYPE 10011
+#define ERROR_GROUP_LOCKED_TYPE 10012
+#define ERROR_GROUP_ADD_ERROR_TYPE 10013
+#define ERROR_NOT_GROUP_OWNER_TYPE 10014
+
 #define JAMMO_NETWORK_BUFFER_MAX_SIZE 1024
 // TODO change to use sha256 -> 32 bytes
 #define JAMMO_MESSAGE_DIGEST_SIZE 20 // 160 bit SHA1
