@@ -814,7 +814,7 @@ void gems_service_group_handle_member_list(gems_components* data, gems_connectio
 					// If we were joining, change state
 					if(data->service_group->group_info->own_state == JOINING) data->service_group->group_info->own_state = IN_GROUP;
 					// inform collaboration about succesful join
-					void collaboration_pair_game_joined(data);
+					collaboration_pair_game_joined(data);
 				}
 				i = i + sizeof(guint32); // to next field
 			}
