@@ -91,8 +91,10 @@ static GstStaticPadTemplate gst_jammo_sampler_src_template =
         "rate = (int) [ 1, MAX ], " "channels = (int) 1")
     );
 
-GST_BOILERPLATE (GstJammoSampler, gst_jammo_sampler, GstBaseSrc,
-    GST_TYPE_BASE_SRC);
+GST_BOILERPLATE (GstJammoSampler, gst_jammo_sampler, GstBaseSrc,GST_TYPE_BASE_SRC)
+
+static void gst_jammo_sampler_dispose (GObject * object);
+static void gst_jammo_sampler_finalize (GObject * object);
 
 static void gst_jammo_sampler_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -153,6 +155,9 @@ static void gst_jammo_sampler_class_init (GstJammoSamplerClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
 
+	gobject_class->dispose = gst_jammo_sampler_dispose;
+	gobject_class->finalize = gst_jammo_sampler_finalize;
+
   gobject_class->set_property = gst_jammo_sampler_set_property;
   gobject_class->get_property = gst_jammo_sampler_get_property;
 
@@ -376,64 +381,9 @@ static void gst_jammo_sampler_src_fixate (GstPad * pad, GstCaps * caps)
     gst_structure_fixate_field_nearest_int (structure, "width", 64);
 }
 
-static JammoSamplerProcessFunc process_funcs[];
 
-static gboolean gst_jammo_sampler_setcaps (GstBaseSrc * basesrc, GstCaps * caps)
-{
-  GstJammoSampler *src = GST_JAMMO_SAMPLER (basesrc);
-  const GstStructure *structure;
-  const gchar *name;
-  gint width;
-  gboolean ret;
 
-  structure = gst_caps_get_structure (caps, 0);
-  ret = gst_structure_get_int (structure, "rate", &src->samplerate);
 
-  GST_DEBUG_OBJECT (src, "negotiated to samplerate %d", src->samplerate);
-
-  name = gst_structure_get_name (structure);
-  if (strcmp (name, "audio/x-raw-int") == 0) {
-    ret &= gst_structure_get_int (structure, "width", &width);
-    src->format = (width == 32) ? GST_JAMMO_SAMPLER_FORMAT_S32 :
-        GST_JAMMO_SAMPLER_FORMAT_S16;
-  } else {
-    ret &= gst_structure_get_int (structure, "width", &width);
-    src->format = (width == 32) ? GST_JAMMO_SAMPLER_FORMAT_F32 :
-        GST_JAMMO_SAMPLER_FORMAT_F64;
-  }
-
-  /* allocate a new buffer suitable for this pad */
-  switch (src->format) {
-    case GST_JAMMO_SAMPLER_FORMAT_S16:
-      printf("format is now gint16\n");
-      src->sample_size = sizeof (gint16);
-      break;
-    case GST_JAMMO_SAMPLER_FORMAT_S32:
-      printf("format is now gint32\n");
-      src->sample_size = sizeof (gint32);
-      break;
-    case GST_JAMMO_SAMPLER_FORMAT_F32:
-      printf("format is now gfloat32\n");
-      src->sample_size = sizeof (gfloat);
-      break;
-    case GST_JAMMO_SAMPLER_FORMAT_F64:
-      printf("format is now gfloat32\n");
-      src->sample_size = sizeof (gdouble);
-      break;
-    default:
-      /* can't really happen */
-      ret = FALSE;
-      break;
-  }
-
-  if (src->format == -1) {
-    src->process = NULL;
-    return FALSE;
-  }
-  src->process = process_funcs[src->format];
-
-  return ret;
-}
 
 static gboolean gst_jammo_sampler_query (GstBaseSrc * basesrc, GstQuery * query)
 {
@@ -499,7 +449,50 @@ error:
   }
 }
 
+static void
+gst_jammo_sampler_dispose (GObject * object)
+{
+  GstJammoSampler *src = GST_JAMMO_SAMPLER (object);
 
+	// free audio data and filenames
+	int i, j;
+	for (i=0;i<JAMMOMIDI_NUMBER_OF_NOTES;i++) {
+		if (src->wav_based_instrument[i]->filename!=NULL) {
+			g_free(src->wav_based_instrument[i]->filename);
+			src->wav_based_instrument[i]->filename=NULL;
+			// if filename was not NULL there may be audio data as well
+			if (src->wav_based_instrument[i]->sampletable!=NULL) {
+				// the same audio data is used for multiple notes
+				// set other pointers as NULL and free one
+				for (j=0;j<JAMMOMIDI_NUMBER_OF_NOTES;j++) {
+					if (i!=j && src->wav_based_instrument[j]!=NULL &&
+						src->wav_based_instrument[j]->filename!=NULL &&
+						src->wav_based_instrument[j]->sampletable!=NULL &&
+						src->wav_based_instrument[i]->sampletable==src->wav_based_instrument[j]->sampletable) {
+							src->wav_based_instrument[j]->sampletable=NULL;
+						}
+				}
+				g_free(src->wav_based_instrument[i]->sampletable);
+				src->wav_based_instrument[i]->sampletable=NULL;
+			}
+		}
+		if (src->wav_based_instrument[i]!=NULL) {
+			g_free(src->wav_based_instrument[i]);
+			src->wav_based_instrument[i]=NULL;
+		}
+	}
+	
+	// free event list
+	jammomidi_free_glist(&(src->eventlist));
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
+static void
+gst_jammo_sampler_finalize (GObject * object)
+{
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
 
 static gfloat looping_sampler(GstJammoSampler * src, gint j) {
 
@@ -519,7 +512,20 @@ static gfloat looping_sampler(GstJammoSampler * src, gint j) {
 		/* if not finished playing */
 		if (*sampleindex < src->wav_based_instrument[j]->file_size) {
 			/* Playing */
-			value=( interpolation(*sampleindex-(gint)*sampleindex, sampletable[(gint)*sampleindex], sampletable[((gint)*sampleindex+1)]) /(G_MAXINT16*1.0) * scale );
+			// do not interpolate if samplerates match
+			if (src->wav_based_instrument[j]->original_samplerate == src->samplerate) {
+				value= sampletable[(gint)*sampleindex] / (G_MAXINT16*1.0) * scale;
+			}
+			else {
+				// samplerates do not match, interpolation needed
+				if ((gint)(*sampleindex+1) >= src->wav_based_instrument[j]->file_size) {
+					// last sample from table is being used. can not interpolate out of sample table
+					value= sampletable[(gint)*sampleindex] / (G_MAXINT16*1.0) * scale;
+				}
+				else {
+					value=( interpolation(*sampleindex-(gint)*sampleindex, sampletable[(gint)*sampleindex], sampletable[((gint)*sampleindex+1)]) /(G_MAXINT16*1.0) * scale );
+				}
+			}
 			/* move sampleindex according to needed pitch */
 			/* samplerate variable has the output samplerate of jammosampler */
 			/* data from sound file is played with correct pitch by interpolating according to file's samplerate */
@@ -605,7 +611,20 @@ static gfloat nonlooping_sampler(GstJammoSampler * src, gint j) {
 		/* if not finished playing */
 		if (*sampleindex < src->wav_based_instrument[j]->file_size) {
 			/* Playing */
-			value=( interpolation(*sampleindex-(gint)*sampleindex, sampletable[(gint)*sampleindex], sampletable[((gint)*sampleindex+1)]) /(G_MAXINT16*1.0) * scale );
+			// do not interpolate if samplerates match
+			if (src->wav_based_instrument[j]->original_samplerate == src->samplerate) {
+				value= sampletable[(gint)*sampleindex] / (G_MAXINT16*1.0) * scale;
+			}
+			else {
+				// samplerates do not match, interpolation needed
+				if ((gint)(*sampleindex+1) >= src->wav_based_instrument[j]->file_size) {
+					// last sample from table is being used. can not interpolate out of sample table
+					value= sampletable[(gint)*sampleindex] / (G_MAXINT16*1.0) * scale;
+				}
+				else {
+					value=( interpolation(*sampleindex-(gint)*sampleindex, sampletable[(gint)*sampleindex], sampletable[((gint)*sampleindex+1)]) /(G_MAXINT16*1.0) * scale );
+				}
+			}
 			/* move sampleindex according to needed pitch */
 			/* samplerate variable has the output samplerate of jammosampler */
 			/* data from sound file is played with correct pitch by interpolating according to file's samplerate */
@@ -673,7 +692,20 @@ static gfloat oneshot_sampler(GstJammoSampler * src, gint j) {
 		/* if not finished playing */
 		if (*sampleindex < src->wav_based_instrument[j]->file_size) {
 			/* Playing */
-			value=( interpolation(*sampleindex-(gint)*sampleindex, sampletable[(gint)*sampleindex], sampletable[((gint)*sampleindex+1)]) /(G_MAXINT16*1.0) * scale );
+			// do not interpolate if samplerates match
+			if (src->wav_based_instrument[j]->original_samplerate == src->samplerate) {
+				value= sampletable[(gint)*sampleindex] / (G_MAXINT16*1.0) * scale;
+			}
+			else {
+				// samplerates do not match, interpolation needed
+				if ((gint)(*sampleindex+1) >= src->wav_based_instrument[j]->file_size) {
+					// last sample from table is being used. can not interpolate out of sample table
+					value= sampletable[(gint)*sampleindex] / (G_MAXINT16*1.0) * scale;
+				}
+				else {
+					value=( interpolation(*sampleindex-(gint)*sampleindex, sampletable[(gint)*sampleindex], sampletable[((gint)*sampleindex+1)]) /(G_MAXINT16*1.0) * scale );
+				}
+			}
 			/* move sampleindex according to needed pitch */
 			/* samplerate variable has the output samplerate of jammosampler */
 			/* data from sound file is played with correct pitch by interpolating according to file's samplerate */
@@ -884,10 +916,10 @@ static void gst_jammo_sampler_process_##type (GstJammoSampler * src, g##type * s
       } \
 }
 
-DEFINE_PROCESS (int16, 32767.0);
-DEFINE_PROCESS (int32, 2147483647.0);
-DEFINE_PROCESS (float, 1.0);
-DEFINE_PROCESS (double, 1.0);
+DEFINE_PROCESS (int16, 32767.0)
+DEFINE_PROCESS (int32, 2147483647.0)
+DEFINE_PROCESS (float, 1.0)
+DEFINE_PROCESS (double, 1.0)
 
 
 static JammoSamplerProcessFunc process_funcs[] = {
@@ -897,6 +929,62 @@ static JammoSamplerProcessFunc process_funcs[] = {
   (JammoSamplerProcessFunc) gst_jammo_sampler_process_double
 };
 
+static gboolean gst_jammo_sampler_setcaps (GstBaseSrc * basesrc, GstCaps * caps)
+{
+  GstJammoSampler *src = GST_JAMMO_SAMPLER (basesrc);
+  const GstStructure *structure;
+  const gchar *name;
+  gint width;
+  gboolean ret;
+
+  structure = gst_caps_get_structure (caps, 0);
+  ret = gst_structure_get_int (structure, "rate", &src->samplerate);
+
+  GST_DEBUG_OBJECT (src, "negotiated to samplerate %d", src->samplerate);
+
+  name = gst_structure_get_name (structure);
+  if (strcmp (name, "audio/x-raw-int") == 0) {
+    ret &= gst_structure_get_int (structure, "width", &width);
+    src->format = (width == 32) ? GST_JAMMO_SAMPLER_FORMAT_S32 :
+        GST_JAMMO_SAMPLER_FORMAT_S16;
+  } else {
+    ret &= gst_structure_get_int (structure, "width", &width);
+    src->format = (width == 32) ? GST_JAMMO_SAMPLER_FORMAT_F32 :
+        GST_JAMMO_SAMPLER_FORMAT_F64;
+  }
+
+  /* allocate a new buffer suitable for this pad */
+  switch (src->format) {
+    case GST_JAMMO_SAMPLER_FORMAT_S16:
+      printf("format is now gint16\n");
+      src->sample_size = sizeof (gint16);
+      break;
+    case GST_JAMMO_SAMPLER_FORMAT_S32:
+      printf("format is now gint32\n");
+      src->sample_size = sizeof (gint32);
+      break;
+    case GST_JAMMO_SAMPLER_FORMAT_F32:
+      printf("format is now gfloat32\n");
+      src->sample_size = sizeof (gfloat);
+      break;
+    case GST_JAMMO_SAMPLER_FORMAT_F64:
+      printf("format is now gfloat32\n");
+      src->sample_size = sizeof (gdouble);
+      break;
+    default:
+      /* can't really happen */
+      ret = FALSE;
+      break;
+  }
+
+  if (src->format == -1) {
+    src->process = NULL;
+    return FALSE;
+  }
+  src->process = process_funcs[src->format];
+
+  return ret;
+}
 
 static void gst_jammo_sampler_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end)
@@ -988,6 +1076,35 @@ static gboolean gst_jammo_sampler_do_seek (GstBaseSrc * basesrc, GstSegment * se
 
   segment->time = segment->start;
   time = segment->last_stop;
+  
+	// seek eventlist to correct position
+	src->time = segment->time;
+	src->playback_index_for_event=0;
+	JammoMidiEvent * event =NULL;
+	while (1) {
+		event=g_list_nth_data(src->eventlist,src->playback_index_for_event);
+		// stop seeking if all events seeked
+		if (event==NULL) {
+			break;
+		}
+		// stop seeking if next event is before current time
+		if (src->time <= event->timestamp) {
+			break;
+		}
+		// no need to stop seeking yet
+		src->playback_index_for_event+=1;
+	}
+	// turn all notes off
+	gint i;
+	for (i=0;i<JAMMOMIDI_NUMBER_OF_NOTES;i++){
+		if (src->wav_based_instrument[i]->filename!=NULL) {
+			src->note_table[i]=FALSE;
+			src->wav_based_instrument[i]->sampleindex=0;
+			src->wav_based_instrument[i]->xfadecounter=-1.0;
+			src->wav_based_instrument[i]->releasecounter=-1.0;
+			src->wav_based_instrument[i]->released=FALSE;
+		}
+	}
 
   /* now move to the time indicated */
   src->next_sample =
@@ -1436,7 +1553,7 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
 		instruments can play c0. Playing notes that are not possible to play was a
 		likely source of segmentation faults with jammo's VI mode. no segfaults after
 		this code was added */
-		if (note>-1 && note < JAMMOMIDI_NUMBER_OF_NOTES) {
+		if (note < JAMMOMIDI_NUMBER_OF_NOTES) {
 			if (src->wav_based_instrument[note]->filename!=NULL) {
 				if (src->note_table[note]==FALSE)
 					src->notes_playing++;
@@ -1453,7 +1570,7 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
 			}
 		}
 	}
-	else if (note>-1 && note < JAMMOMIDI_NUMBER_OF_NOTES) { //JAMMOMIDI_NOTE_OFF
+	else if (note < JAMMOMIDI_NUMBER_OF_NOTES) { //JAMMOMIDI_NOTE_OFF
 		// release note only, the number of notes will be decreased when the note has faded
 		src->wav_based_instrument[note]->released=TRUE;
 	}
@@ -1494,6 +1611,9 @@ gst_jammo_sampler_set_property (GObject * object, guint prop_id,
       break;
     case PROP_RECORDING:
       src->recording = g_value_get_boolean (value);
+			// when playbacking sampler needs to be not is-live so that seeking works better
+			// when recording sampler needs to be is-live for smoother response
+      gst_base_src_set_live (GST_BASE_SRC (src), g_value_get_boolean (value));
       break;
    case PROP_EVENTLIST:
       //We got GList containing JammoMidiEvents, take all one-by-one and add to own internal eventlist
