@@ -101,14 +101,8 @@ static void jammo_instrument_track_finalize(GObject* object) {
 }
 
 static void jammo_instrument_track_dispose(GObject* object) {
-	JammoInstrumentTrack* instrument_track;
-	GList * list;
-
-	instrument_track = JAMMO_INSTRUMENT_TRACK(object);
-	// free the event list of sampler
-	list = jammo_instrument_track_get_event_list(instrument_track);
-	jammomidi_free_glist(&list);
-
+	JammoInstrumentTrack * instrument_track = JAMMO_INSTRUMENT_TRACK(object);
+	g_object_unref(instrument_track->priv->element);
 	G_OBJECT_CLASS(jammo_instrument_track_parent_class)->dispose(object);
 }
 
@@ -138,6 +132,8 @@ static GObject* jammo_instrument_track_constructor(GType type, guint n_propertie
 	printf ("Type of instrument is: %d\n",instrument_nro);
 	
 	instrument_track->priv->element = gst_element_factory_make("jammosampler", NULL);
+	// instrument track has a reference to element so refcount needs to be increased
+	g_object_ref(instrument_track->priv->element);
 
 	//We must first define folder of samples
 	g_object_set(G_OBJECT(instrument_track->priv->element), "instrument-folder", INSTRUMENT_FOLDER, NULL);
@@ -156,17 +152,7 @@ static GObject* jammo_instrument_track_constructor(GType type, guint n_propertie
 
 	g_object_set(G_OBJECT(instrument_track->priv->element), "init-instrument", buffer, NULL);
 
-	g_object_set(G_OBJECT(instrument_track->priv->element), "realtime", TRUE, NULL);
-
-	// first note used to play for too long if pipeline was not live
-	// TODO add a parameter to constructor that defines whether instrument track is live or not
-	// in real time pair jamming live instrument track should not be used because pipeline needs
-	// to be seeked to synchronize pipelines on different devices. otherwise instrument tracks
-	// should be live for more stable playback
-
-	// NOTE: it seems that seeking is not feasible anyway, so there is no need to change anything
-	// at least yet
-	g_object_set(G_OBJECT(instrument_track->priv->element), "is-live", TRUE, NULL);
+	g_object_set(G_OBJECT(instrument_track->priv->element), "realtime", FALSE, NULL);
 
 	return object;
 }
