@@ -90,6 +90,9 @@ static void jammo_slider_track_dispose(GObject* object) {
 	JammoSliderTrack* slider_track;
 
 	slider_track = JAMMO_SLIDER_TRACK(object);
+	g_object_unref(slider_track->priv->bin);
+	g_object_unref(slider_track->priv->slider);
+	g_object_unref(slider_track->priv->filter);
 	G_OBJECT_CLASS(jammo_slider_track_parent_class)->dispose(object);
 }
 
@@ -118,9 +121,15 @@ static GObject* jammo_slider_track_constructor(GType type, guint n_properties, G
 	printf ("Type of slider is: %d\n",slider_nro);
 	
 	slider_track->priv->bin = gst_bin_new("slider-track");
+	// slider track has a reference to bin so refcount needs to be increased
+	g_object_ref(slider_track->priv->bin);
 
 	slider_track->priv->slider = gst_element_factory_make("jammoslider", NULL);
+	// slider track has a reference to slider
+	g_object_ref(slider_track->priv->slider);
 	slider_track->priv->filter = gst_element_factory_make("audiocheblimit", NULL);
+	// slider track has a reference to filter
+	g_object_ref(slider_track->priv->filter);
 	g_object_set(G_OBJECT(slider_track->priv->slider), "instrument", slider_nro, NULL);
 
 	if (slider_nro == JAMMO_SLIDER_TYPE_KARPLUS) {
@@ -141,7 +150,7 @@ static GObject* jammo_slider_track_constructor(GType type, guint n_properties, G
 			g_object_set(slider_track->priv->filter, "cutoff", 1000.0, NULL);
 	}
 	
-	g_object_set(G_OBJECT(slider_track->priv->slider), "recording", TRUE, NULL);
+	g_object_set(G_OBJECT(slider_track->priv->slider), "recording", FALSE, NULL);
 	
 	gst_bin_add_many(GST_BIN(slider_track->priv->bin), slider_track->priv->slider, slider_track->priv->filter, NULL);
 	gst_element_link(slider_track->priv->slider, slider_track->priv->filter);
