@@ -59,25 +59,6 @@ guint64 jammo_track_get_duration(JammoTrack* track) {
 	return duration;
 }
 
-
-/**
- * jammo_track_set_muted:
- * @track: a #JammoTrack
- * @muted: whether the track is muted or not
- *
- * A track can be muted temporarily with this function.
- *
- * Implemented in subclasses.
- */
-void jammo_track_set_muted(JammoTrack* track, gboolean muted) {
-	JammoTrackClass* track_class;
-	
-	track_class = JAMMO_TRACK_GET_CLASS(track);
-	if (track_class->set_muted) {
-		track_class->set_muted(track, muted);
-	}
-}
-
 /**
  * _jammo_track_get_element:
  * @track: a #JammoTrack
