@@ -77,8 +77,9 @@ int main (int argc, char **argv) {
 		printf("problem in getting and setting socket\n");
 	}
 
-	printf("starting\n");
-	jammo_collaboration_pair_composition_start(collaboration_pair_composition, FALSE, 0, 0);
+	// can not start. it requires gems
+	//printf("starting\n");
+	//jammo_collaboration_pair_composition_start(collaboration_pair_composition, FALSE, 0, 0);
 	
 	jammo_collaboration_game_teacher_exit(JAMMO_COLLABORATION_GAME(collaboration_pair_composition));
 	jammo_collaboration_game_create_song_file(JAMMO_COLLABORATION_GAME(collaboration_pair_composition));
